"""Return Weather and Time from API's."""

from time import localtime, sleep

cities = {'AMSTERDAM': ['52.37', '4.90', 'Europe/Amsterdam'],
          'ATHENS': ['37.98', '23.73', 'Europe/Athens'],
          'AUCKLAND': ['-36.85', '174.77', 'Pacific/Auckland'],
          'BANGKOK': ['13.76', '100.50', 'Asia/Bangkok'],
          'BERLIN': ['52.52', '13.41', 'Europe/Berlin'],
          'BERN': ['46.95', '7.45', 'Europe/Zurich'],
          'BOGOTA': ['4.60', '-74.07', 'America/Bogota'],
          'BRATISLAVA': ['48.14', '17.10', 'Etc/GMT-2'],
          'BRUSSELS': ['50.85', '4.35', 'Europe/Brussels'],
          'BUCHAREST': ['44.42', '26.10', 'Europe/Bucharest'],
          'BUDAPEST': ['47.50', '19.04', 'Europe/Budapest'],
          'CAPE TOWN': ['-33.93', '18.42', 'Africa/Johannesburg'],
          'CHICAGO': ['41.88', '-87.63', 'America/Chicago'],
          'DHAKA': ['23.81', '90.41', 'Asia/Dhaka'],
          'DUBAI': ['25.20', '55.30', 'Asia/Dubai'],
          'DUBLIN': ['53.34', '-6.26', 'Europe/Dublin'],
          'FARGO': ['46.88', '-96.79', 'America/North_Dakota/Center'],
          'HANOI': ['23.09', '-82.36', 'Asia/Ho_Chi_Minh'],
          'HAVANA': ['21.03', '105.84', 'America/Havana'],
          'HELSINKI': ['60.17', '24.94', 'Europe/Helsinki'],
          'ISTANBUL': ['41.01', '28.98', 'Europe/Istanbul'],
          'JAKARTA': ['-6.21', '106.85', 'Asia/Jakarta'],
          'JERUSALEM': ['31.77', '35.21', 'Asia/Jerusalem'],
          'KATHMANDU': ['27.70', '85.31', 'Asia/Kathmandu'],
          'KUALA LUMPUR': ['3.15', '101.69', 'Asia/Kuala_Lumpur'],
          'KIEV': ['50.44', '30.51', 'Europe/Kiev'],
          'LIMA': ['-12.04', '-77.04', 'America/Lima'],
          'LISBON': ['38.73', '-9.14', 'Europe/Lisbon'],
          'LONDON': ['51.51', '-0.13', 'Europe/London'],
          'LOS ANGELES': ['34.09', '-118.31', 'America/Los_Angeles'],
          'MADRID': ['40.41', '-3.70', 'Europe/Madrid'],
          'MANILA': ['14.60', '120.98', 'Asia/Manila'],
          'MCMURDO': ['-77.84', '166.68', 'Pacific/Auckland'],
          'MELBOURNE': ['-37.81', '144.96', 'Australia/Melbourne'],
          'MEXICO CITY': ['19.43', '-99.14', 'America/Mexico_City'],
          'MONTREAL': ['45.50', '-73.57', 'EST'],
          'MOSCOW': ['55.76', '37.59', 'Europe/Moscow'],
          'MUMBAI': ['19.09', '72.88', 'Asia/Colombo'],
          'NEW YORK': ['40.71', '-74.01', 'America/New_York'],
          'OSLO': ['59.91', '10.75', 'Europe/Oslo'],
          'PARIS': ['48.85', '2.35', 'Europe/Paris'],
          'PRAGUE': ['50.08', '14.42', 'Europe/Prague'],
          'REYKJAVIK': ['64.15', '-21.92', 'Atlantic/Reykjavik'],
          'RIO': ['-22.91', '-43.17', 'Etc/GMT+3'],
          'ROME': ['41.89', '12.48', 'Europe/Rome'],
          'RIYAHD': ['24.71', '46.67', 'Asia/Riyadh'],
          'SAN FRANCISCO': ['37.80', '-122.40', 'PST8PDT'],
          'SAN JUAN': ['18.43', '-66.09', 'America/Puerto_Rico'],
          'SANTIAGO': ['-33.45', '-70.68', 'America/Santiago'],
          'SEATTLE': ['47.57', '-122.38', 'PST8PDT'],
          'SEOUL': ['37.57', '126.98', 'Asia/Seoul'],
          'SHANGHAI': ['31.22', '121.47', 'Asia/Shanghai'],
          'SINGAPORE': ['1.35', '103.85', 'Asia/Singapore'],
          'SOFIA': ['42.70', '23.32', 'Europe/Sofia'],
          'STOCKHOLM': ['59.33', '18.07', 'Europe/Stockholm'],
          'TAIPEI': ['25.03', '121.56', 'Asia/Taipei'],
          'TOKYO': ['35.68', '139.64', 'Asia/Tokyo'],
          'TUCUMAN': ['-26.81', '-65.22', 'America/Argentina/Tucuman'],
          'VALLETTA': ['35.90', '15.51', 'Europe/Malta'],
          'VIENNA': ['48.21', '16.38', 'Europe/Vienna'],
          'WARSAW': ['52.23', '21.01', 'Europe/Warsaw'],
          'ZAGREB': ['45.82', '15.98', 'Etc/GMT-2']}

icon_frames = {'1499366020783': 0,
               '1499366020844': 1,
               '1499366020890': 2,
               '1499366020911': 2,
               '1499366020964': 3,
               '1499366020983': 3,
               '1499366021170': 4,
               '1499366021215': 4,
               '1499366021284': 5,
               '1499366021399': 6,
               '1499366021485': 7,
               '1499366021608': 7,
               '1499366021687': 8,
               '1499366021722': 8,
               '1499366021821': 9,
               '1499366021771': 10,
               '1499366021876': 10,
               '1499366022009': 11}


def call_api(url, wifi):
    """Call web API.

    Args:
        url (string): URL of API.
        wifi (class): WiFi manager.
    """

    headers = {'Pragma': 'no-cache', 'Cache-Control': 'no-cache'}
    while True:
        try:
            # Call Free Code Camp Weather API
            response = wifi.get(url, headers=headers)
            result = response.json()
            status_code = response.status_code
            response.close()
            # Only return result if status code is OK
            if status_code == 200:
                return result
            print('data unavailable...')
            sleep(2)
        except (ValueError, RuntimeError) as e:
            print("Failed to get data, retrying\n", e)
            wifi.reset()
            continue


def getWeatherConditions(city, wifi):
    """Get weather data from FCC weather API.

    Args:
        city (string): Name of city to get weather.
        wifi (class): WiFi manager.
    """

    lat, lon, tz = cities[city]
    url = 'https://fcc-weather-api.glitch.me/api/current?lat={0}&lon={1}'.\
        format(lat, lon)

    # Get weather data from API
    while True:
        data = call_api(url, wifi)
        # If the weather API fails it returns Shuzenji
        if data['name'] != 'Shuzenji':
            break
        sleep(2)  # Sleep and try again

    dt = int(data['dt'])
    sunrise = int(data['sys']['sunrise'])
    sunset = int(data['sys']['sunset'])

    # Call World Time API to get timezone offset
    url = 'http://worldtimeapi.org/api/timezone/{0}'.format(tz)
    data_time = call_api(url, wifi)

    # Adjust time, sunset and sunrise to time zone
    if isinstance(data_time, dict) and 'error' not in data_time.keys():
        offset_hours, offset_minutes = data_time['utc_offset'].split(':')
        utc_offset = (int(offset_hours) * 3600) + (int(offset_minutes) * 60)
        dt = int(data_time['unixtime'])  # API time is more accurate
        dt += utc_offset
        sunrise += utc_offset
        sunset += utc_offset
    else:
        print('Failed to get time zone offset.')

    # Convert unix time to struct_time
    dt = localtime(dt)
    sunrise = localtime(sunrise)
    sunset = localtime(sunset)

    # Get weather condition icon sprite frame
    if 'icon' in data['weather'][0]:
        icon = data['weather'][0]['icon']
        suffix = icon[icon.rfind("?") + 1:]
        frame = icon_frames[suffix]
    else:
        frame = 12

    return {'condition': data['weather'][0]['description'],
            'datetime': dt,
            'humidity': data['main']['humidity'],
            'icon_frame': frame,
            'sunrise': '{0:02}:{1:02}'.format(sunrise.tm_hour, sunrise.tm_min),
            'sunset': '{0:02}:{1:02}'.format(sunset.tm_hour, sunset.tm_min),
            'temperature': data['main']['temp'],
            'wind': data['wind']['speed'] * 3.6}